@@ -26,12 +26,11 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"github.com/rancher/k3d/v4/cmd/util"
-	"github.com/rancher/k3d/v4/pkg/runtimes"
-	"github.com/rancher/k3d/v4/pkg/tools"
-	k3d "github.com/rancher/k3d/v4/pkg/types"
-
-	log "github.com/sirupsen/logrus"
+	"github.com/rancher/k3d/v5/cmd/util"
+	"github.com/rancher/k3d/v5/pkg/client"
+	l "github.com/rancher/k3d/v5/pkg/logger"
+	"github.com/rancher/k3d/v5/pkg/runtimes"
+	k3d "github.com/rancher/k3d/v5/pkg/types"
 )
 
 // NewCmdImageImport returns a new cobra command
@@ -56,24 +55,24 @@ That is, 'rancher/k3d-tools' is treated as 'rancher/k3d-tools:latest'.
 
 A file ARCHIVE always takes precedence.
 So if a file './rancher/k3d-tools' exists, k3d will try to import it instead of the IMAGE of the same name.`,
-		Aliases: []string{"images"},
+		Aliases: []string{"load"},
 		Args:    cobra.MinimumNArgs(1),
 		Run: func(cmd *cobra.Command, args []string) {
 			images, clusters := parseLoadImageCmd(cmd, args)
-			log.Debugf("Importing image(s) [%+v] from runtime [%s] into cluster(s) [%+v]...", images, runtimes.SelectedRuntime, clusters)
+			l.Log().Debugf("Importing image(s) [%+v] from runtime [%s] into cluster(s) [%+v]...", images, runtimes.SelectedRuntime, clusters)
 			errOccured := false
 			for _, cluster := range clusters {
-				log.Infof("Importing image(s) into cluster '%s'", cluster.Name)
-				if err := tools.ImageImportIntoClusterMulti(cmd.Context(), runtimes.SelectedRuntime, images, &cluster, loadImageOpts); err != nil {
-					log.Errorf("Failed to import image(s) into cluster '%s': %+v", cluster.Name, err)
+				l.Log().Infof("Importing image(s) into cluster '%s'", cluster.Name)
+				if err := client.ImageImportIntoClusterMulti(cmd.Context(), runtimes.SelectedRuntime, images, &cluster, loadImageOpts); err != nil {
+					l.Log().Errorf("Failed to import image(s) into cluster '%s': %+v", cluster.Name, err)
 					errOccured = true
 				}
 			}
 			if errOccured {
-				log.Warnln("At least one error occured while trying to import the image(s) into the selected cluster(s)")
+				l.Log().Warnln("At least one error occured while trying to import the image(s) into the selected cluster(s)")
 				os.Exit(1)
 			}
-			log.Infof("Successfully imported %d image(s) into %d cluster(s)", len(images), len(clusters))
+			l.Log().Infof("Successfully imported %d image(s) into %d cluster(s)", len(images), len(clusters))
 		},
 	}
 
@@ -82,7 +81,7 @@ So if a file './rancher/k3d-tools' exists, k3d will try to import it instead of
 	 *********/
 	cmd.Flags().StringArrayP("cluster", "c", []string{k3d.DefaultClusterName}, "Select clusters to load the image to.")
 	if err := cmd.RegisterFlagCompletionFunc("cluster", util.ValidArgsAvailableClusters); err != nil {
-		log.Fatalln("Failed to register flag completion for '--cluster'", err)
+		l.Log().Fatalln("Failed to register flag completion for '--cluster'", err)
 	}
 
 	cmd.Flags().BoolVarP(&loadImageOpts.KeepTar, "keep-tarball", "k", false, "Do not delete the tarball containing the saved images from the shared volume")
@@ -100,7 +99,7 @@ func parseLoadImageCmd(cmd *cobra.Command, args []string) ([]string, []k3d.Clust
 	// --cluster
 	clusterNames, err := cmd.Flags().GetStringArray("cluster")
 	if err != nil {
-		log.Fatalln(err)
+		l.Log().Fatalln(err)
 	}
 	clusters := []k3d.Cluster{}
 	for _, clusterName := range clusterNames {
@@ -110,7 +109,7 @@ func parseLoadImageCmd(cmd *cobra.Command, args []string) ([]string, []k3d.Clust
 	// images
 	images := args
 	if len(images) == 0 {
-		log.Fatalln("No images specified!")
+		l.Log().Fatalln("No images specified!")
 	}
 
 	return images, clusters
