@@ -22,13 +22,16 @@ THE SOFTWARE.
 package client
 
 import (
+	"bytes"
 	"context"
 	_ "embed"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
+	"os"
 	"sort"
 	"strconv"
+	"strings"
 	"time"
 
 	gort "runtime"
@@ -86,6 +89,7 @@ func ClusterRun(ctx context.Context, runtime k3drt.Runtime, clusterConfig *confi
 		Timeout:         clusterConfig.ClusterCreateOpts.Timeout, // TODO: here we should consider the time used so far
 		NodeHooks:       clusterConfig.ClusterCreateOpts.NodeHooks,
 		EnvironmentInfo: envInfo,
+		Intent:          k3d.IntentClusterCreate,
 	}); err != nil {
 		return fmt.Errorf("Failed Cluster Start: %+v", err)
 	}
@@ -376,7 +380,7 @@ ClusterCreatOpts:
 	// connection url is always the name of the first server node (index 0) // TODO: change this to the server loadbalancer
 	connectionURL := fmt.Sprintf("https://%s:%s", GenerateNodeName(cluster.Name, k3d.ServerRole, 0), k3d.DefaultAPIPort)
 	clusterCreateOpts.GlobalLabels[k3d.LabelClusterURL] = connectionURL
-	clusterCreateOpts.GlobalEnv = append(clusterCreateOpts.GlobalEnv, fmt.Sprintf("%s=%s", k3d.K3sEnvClusterToken, cluster.Token))
+	clusterCreateOpts.GlobalEnv = append(clusterCreateOpts.GlobalEnv, fmt.Sprintf("%s=%s", k3s.EnvClusterToken, cluster.Token))
 
 	nodeSetup := func(node *k3d.Node) error {
 		// cluster specific settings
@@ -410,12 +414,12 @@ ClusterCreatOpts:
 
 			// the cluster has an init server node, but its not this one, so connect it to the init node
 			if cluster.InitNode != nil && !node.ServerOpts.IsInit {
-				node.Env = append(node.Env, fmt.Sprintf("%s=%s", k3d.K3sEnvClusterConnectURL, connectionURL))
+				node.Env = append(node.Env, fmt.Sprintf("%s=%s", k3s.EnvClusterConnectURL, connectionURL))
 				node.RuntimeLabels[k3d.LabelServerIsInit] = "false" // set label, that this server node is not the init server
 			}
 
 		} else if node.Role == k3d.AgentRole {
-			node.Env = append(node.Env, fmt.Sprintf("%s=%s", k3d.K3sEnvClusterConnectURL, connectionURL))
+			node.Env = append(node.Env, fmt.Sprintf("%s=%s", k3s.EnvClusterConnectURL, connectionURL))
 		}
 
 		node.Networks = []string{cluster.Network.Name}
@@ -819,6 +823,10 @@ func GenerateNodeName(cluster string, role k3d.Role, suffix int) string {
 func ClusterStart(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster, clusterStartOpts types.ClusterStartOpts) error {
 	l.Log().Infof("Starting cluster '%s'", cluster.Name)
 
+	if clusterStartOpts.Intent == "" {
+		clusterStartOpts.Intent = k3d.IntentClusterStart
+	}
+
 	if clusterStartOpts.Timeout > 0*time.Second {
 		var cancel context.CancelFunc
 		ctx, cancel = context.WithTimeout(ctx, clusterStartOpts.Timeout)
@@ -844,18 +852,10 @@ func ClusterStart(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Clust
 		}
 	}
 
-	// TODO: remove trace logs below
-	l.Log().Traceln("Servers before sort:")
-	for i, n := range servers {
-		l.Log().Tracef("Server %d - %s", i, n.Name)
-	}
+	// sort list of servers for properly ordered sequential start
 	sort.Slice(servers, func(i, j int) bool {
 		return servers[i].Name < servers[j].Name
 	})
-	l.Log().Traceln("Servers after sort:")
-	for i, n := range servers {
-		l.Log().Tracef("Server %d - %s", i, n.Name)
-	}
 
 	/*
 	 * Init Node
@@ -865,7 +865,7 @@ func ClusterStart(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Clust
 		if err := NodeStart(ctx, runtime, initNode, &k3d.NodeStartOpts{
 			Wait:            true, // always wait for the init node
 			NodeHooks:       clusterStartOpts.NodeHooks,
-			ReadyLogMessage: "Running kube-apiserver", // initNode means, that we're using etcd -> this will need quorum, so "k3s is up and running" won't happen right now
+			ReadyLogMessage: types.GetReadyLogMessage(initNode, clusterStartOpts.Intent), // initNode means, that we're using etcd -> this will need quorum, so "k3s is up and running" won't happen right now
 			EnvironmentInfo: clusterStartOpts.EnvironmentInfo,
 		}); err != nil {
 			return fmt.Errorf("Failed to start initializing server node: %+v", err)
@@ -879,7 +879,7 @@ func ClusterStart(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Clust
 	for _, serverNode := range servers {
 		if err := NodeStart(ctx, runtime, serverNode, &k3d.NodeStartOpts{
 			Wait:            true,
-			NodeHooks:       clusterStartOpts.NodeHooks,
+			NodeHooks:       append(clusterStartOpts.NodeHooks, serverNode.HookActions...),
 			EnvironmentInfo: clusterStartOpts.EnvironmentInfo,
 		}); err != nil {
 			return fmt.Errorf("Failed to start server %s: %+v", serverNode.Name, err)
@@ -934,19 +934,82 @@ func ClusterStart(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Clust
 	}
 
 	/*
-	 * Additional Cluster Preparation
+	 * Additional Cluster Preparation (post start)
 	 */
 
+	postStartErrgrp, postStartErrgrpCtx := errgroup.WithContext(ctx)
+
 	/*** DNS ***/
 
-	// add /etc/hosts and CoreDNS entry for host.k3d.internal, referring to the host system
-	if err := prepInjectHostIP(ctx, runtime, cluster, &clusterStartOpts); err != nil {
-		return fmt.Errorf("failed to inject host IP: %w", err)
-	}
+	// add host.k3d.internal record to /etc/hosts in all nodes
+	postStartErrgrp.Go(func() error {
+		return prepInjectHostIP(postStartErrgrpCtx, runtime, cluster, &clusterStartOpts)
+	})
 
-	// create host records in CoreDNS for external registries
-	if err := prepCoreDNSInjectNetworkMembers(ctx, runtime, cluster); err != nil {
-		return fmt.Errorf("failed to patch CoreDNS with network members: %w", err)
+	postStartErrgrp.Go(func() error {
+
+		hosts := fmt.Sprintf("%s %s\n", clusterStartOpts.EnvironmentInfo.HostGateway.String(), k3d.DefaultK3dInternalHostRecord)
+
+		net, err := runtime.GetNetwork(ctx, &cluster.Network)
+		if err != nil {
+			return fmt.Errorf("failed to get cluster network %s to inject host records into CoreDNS: %w", cluster.Network.Name, err)
+		}
+		for _, member := range net.Members {
+			hosts += fmt.Sprintf("%s %s\n", member.IP.String(), member.Name)
+		}
+
+		l.Log().Infof("Injecting records for host.k3d.internal and for %d network members into CoreDNS configmap...", len(net.Members))
+		act := actions.RewriteFileAction{
+			Runtime: runtime,
+			Path:    "/var/lib/rancher/k3s/server/manifests/coredns.yaml",
+			Mode:    0744,
+			RewriteFunc: func(input []byte) ([]byte, error) {
+				split, err := util.SplitYAML(input)
+				if err != nil {
+					return nil, fmt.Errorf("error splitting yaml: %w", err)
+				}
+
+				var outputBuf bytes.Buffer
+				outputEncoder := yaml.NewEncoder(&outputBuf)
+
+				for _, d := range split {
+					var doc map[string]interface{}
+					if err := yaml.Unmarshal(d, &doc); err != nil {
+						return nil, err
+					}
+					if kind, ok := doc["kind"]; ok {
+						if strings.ToLower(kind.(string)) == "configmap" {
+							configmapData := doc["data"].(map[interface{}]interface{})
+							configmapData["NodeHosts"] = hosts
+						}
+					}
+					if err := outputEncoder.Encode(doc); err != nil {
+						return nil, err
+					}
+				}
+				outputEncoder.Close()
+				return outputBuf.Bytes(), nil
+			},
+		}
+
+		// get the first server in the list and run action on it once it's ready for it
+		for _, n := range cluster.Nodes {
+			if n.Role == k3d.ServerRole {
+				ts, err := time.Parse("2006-01-02T15:04:05.999999999Z", n.State.Started)
+				if err != nil {
+					return err
+				}
+				if err := NodeWaitForLogMessage(ctx, runtime, n, "Cluster dns configmap", ts.Truncate(time.Second)); err != nil {
+					return err
+				}
+				return act.Run(ctx, n)
+			}
+		}
+		return nil
+	})
+
+	if err := postStartErrgrp.Wait(); err != nil {
+		return fmt.Errorf("error during post-start cluster preparation: %w", err)
 	}
 
 	return nil
@@ -983,30 +1046,55 @@ func SortClusters(clusters []*k3d.Cluster) []*k3d.Cluster {
 
 // corednsAddHost adds a host entry to the CoreDNS configmap if it doesn't exist (a host entry is a single line of the form "IP HOST")
 func corednsAddHost(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster, ip string, name string) error {
+	retries := 3
+	if v, ok := os.LookupEnv(k3d.K3dEnvDebugCorednsRetries); ok && v != "" {
+		l.Log().Debugf("Running with %s=%s", k3d.K3dEnvDebugCorednsRetries, v)
+		if r, err := strconv.Atoi(v); err == nil {
+			retries = r
+		} else {
+			return fmt.Errorf("Invalid value set for env var %s (%s): %w", k3d.K3dEnvDebugCorednsRetries, v, err)
+		}
+	}
+
+	// select any server node
+	var node *k3d.Node
+	for _, n := range cluster.Nodes {
+		if n.Role == k3d.ServerRole {
+			node = n
+		}
+	}
+
 	hostsEntry := fmt.Sprintf("%s %s", ip, name)
 	patchCmd := `patch=$(kubectl get cm coredns -n kube-system --template='{{.data.NodeHosts}}' | sed -n -E -e '/[0-9\.]{4,12}\s` + name + `$/!p' -e '$a` + hostsEntry + `' | tr '\n' '^' | busybox xargs -0 printf '{"data": {"NodeHosts":"%s"}}'| sed -E 's%\^%\\n%g') && kubectl patch cm coredns -n kube-system -p="$patch"`
 	successInjectCoreDNSEntry := false
-	for _, node := range cluster.Nodes {
-		if node.Role == k3d.AgentRole || node.Role == k3d.ServerRole {
-			logreader, err := runtime.ExecInNodeGetLogs(ctx, node, []string{"sh", "-c", patchCmd})
-			if err == nil {
-				successInjectCoreDNSEntry = true
-				break
-			} else {
-				msg := fmt.Sprintf("error patching the CoreDNS ConfigMap to include entry '%s': %+v", hostsEntry, err)
-				readlogs, err := ioutil.ReadAll(logreader)
+
+	// try 3 (or K3D_DEBUG_COREDNS_RETRIES value) times, as e.g. on cluster startup it may take some time for the Configmap to be available and the server to be responsive
+	for i := 0; i < retries; i++ {
+		l.Log().Debugf("Running CoreDNS patch in node %s to add %s (try %d/%d)...", node.Name, hostsEntry, i, retries)
+		logreader, err := runtime.ExecInNodeGetLogs(ctx, node, []string{"sh", "-c", patchCmd})
+		if err == nil {
+			successInjectCoreDNSEntry = true
+			break
+		} else {
+			msg := fmt.Sprintf("(try %d/%d) error patching the CoreDNS ConfigMap to include entry '%s': %+v", i, retries, hostsEntry, err)
+			if logreader != nil {
+				readlogs, err := io.ReadAll(logreader)
 				if err != nil {
-					l.Log().Debugf("error reading the logs from failed CoreDNS patch exec process in node %s: %v", node.Name, err)
+					l.Log().Debugf("(try %d/%d) error reading the logs from failed CoreDNS patch exec process in node %s: %v", i, retries, node.Name, err)
 				} else {
 					msg += fmt.Sprintf("\nLogs: %s", string(readlogs))
 				}
-				l.Log().Debugln(msg)
+			} else {
+				l.Log().Debugf("(try %d/%d) error reading the logs from failed CoreDNS patch exec process in node %s: no logreader returned for exec process", i, retries, node.Name)
 			}
+			l.Log().Debugln(msg)
+			time.Sleep(1 * time.Second)
 		}
 	}
 	if !successInjectCoreDNSEntry {
-		return fmt.Errorf("Failed to patch CoreDNS ConfigMap to include entry '%s' (see debug logs)", hostsEntry)
+		return fmt.Errorf("failed to patch CoreDNS ConfigMap to include entry '%s' (%d tries, see debug logs)", hostsEntry, retries)
 	}
+	l.Log().Debugf("Successfully patched CoreDNS Configmap with record '%s'", hostsEntry)
 	return nil
 }
 
@@ -1019,7 +1107,7 @@ func prepInjectHostIP(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.C
 
 	hostIP := clusterStartOpts.EnvironmentInfo.HostGateway
 	hostsEntry := fmt.Sprintf("%s %s", hostIP.String(), k3d.DefaultK3dInternalHostRecord)
-	l.Log().Infof("Injecting record '%s'...", hostsEntry)
+	l.Log().Infof("Injecting '%s' into /etc/hosts of all nodes...", hostsEntry)
 
 	// entry in /etc/hosts
 	errgrp, errgrpctx := errgroup.WithContext(ctx)
@@ -1034,26 +1122,6 @@ func prepInjectHostIP(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.C
 	}
 	l.Log().Debugf("Successfully added host record \"%s\" to /etc/hosts in all nodes", hostsEntry)
 
-	err := corednsAddHost(ctx, runtime, cluster, hostIP.String(), k3d.DefaultK3dInternalHostRecord)
-	if err != nil {
-		return fmt.Errorf("failed to inject host record \"%s\" into CoreDNS ConfigMap: %w", hostsEntry, err)
-	}
-	l.Log().Debugf("Successfully added host record \"%s\" to the CoreDNS ConfigMap ", hostsEntry)
-	return nil
-}
-
-func prepCoreDNSInjectNetworkMembers(ctx context.Context, runtime k3drt.Runtime, cluster *k3d.Cluster) error {
-	net, err := runtime.GetNetwork(ctx, &cluster.Network)
-	if err != nil {
-		return fmt.Errorf("failed to get cluster network %s to inject host records into CoreDNS: %v", cluster.Network.Name, err)
-	}
-	l.Log().Debugf("Adding %d network members to coredns", len(net.Members))
-	for _, member := range net.Members {
-		hostsEntry := fmt.Sprintf("%s %s", member.IP.String(), member.Name)
-		if err := corednsAddHost(ctx, runtime, cluster, member.IP.String(), member.Name); err != nil {
-			return fmt.Errorf("failed to add host entry \"%s\" into CoreDNS: %v", hostsEntry, err)
-		}
-	}
 	return nil
 }
 
