@@ -57,23 +57,23 @@ var DefaultConfig = fmt.Sprintf(
 )
 
 type VolumeWithNodeFilters struct {
-	Volume      string   `mapstructure:"volume" yaml:"volume" json:"volume,omitempty"`
-	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters" json:"nodeFilters,omitempty"`
+	Volume      string   `mapstructure:"volume" yaml:"volume,omitempty" json:"volume,omitempty"`
+	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters,omitempty" json:"nodeFilters,omitempty"`
 }
 
 type PortWithNodeFilters struct {
-	Port        string   `mapstructure:"port" yaml:"port" json:"port,omitempty"`
-	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters" json:"nodeFilters,omitempty"`
+	Port        string   `mapstructure:"port" yaml:"port,omitempty" json:"port,omitempty"`
+	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters,omitempty" json:"nodeFilters,omitempty"`
 }
 
 type LabelWithNodeFilters struct {
-	Label       string   `mapstructure:"label" yaml:"label" json:"label,omitempty"`
-	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters" json:"nodeFilters,omitempty"`
+	Label       string   `mapstructure:"label" yaml:"label,omitempty" json:"label,omitempty"`
+	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters,omitempty" json:"nodeFilters,omitempty"`
 }
 
 type EnvVarWithNodeFilters struct {
-	EnvVar      string   `mapstructure:"envVar" yaml:"envVar" json:"envVar,omitempty"`
-	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters" json:"nodeFilters,omitempty"`
+	EnvVar      string   `mapstructure:"envVar" yaml:"envVar,omitempty" json:"envVar,omitempty"`
+	NodeFilters []string `mapstructure:"nodeFilters" yaml:"nodeFilters,omitempty" json:"nodeFilters,omitempty"`
 }
 
 // SimpleConfigOptionsKubeconfig describes the set of options referring to the kubeconfig during cluster creation.
@@ -83,49 +83,49 @@ type SimpleConfigOptionsKubeconfig struct {
 }
 
 type SimpleConfigOptions struct {
-	K3dOptions        SimpleConfigOptionsK3d        `mapstructure:"k3d" yaml:"k3d"`
-	K3sOptions        SimpleConfigOptionsK3s        `mapstructure:"k3s" yaml:"k3s"`
-	KubeconfigOptions SimpleConfigOptionsKubeconfig `mapstructure:"kubeconfig" yaml:"kubeconfig"`
-	Runtime           SimpleConfigOptionsRuntime    `mapstructure:"runtime" yaml:"runtime"`
+	K3dOptions        SimpleConfigOptionsK3d        `mapstructure:"k3d" yaml:"k3d" json:"k3d"`
+	K3sOptions        SimpleConfigOptionsK3s        `mapstructure:"k3s" yaml:"k3s" json:"k3s"`
+	KubeconfigOptions SimpleConfigOptionsKubeconfig `mapstructure:"kubeconfig" yaml:"kubeconfig" json:"kubeconfig"`
+	Runtime           SimpleConfigOptionsRuntime    `mapstructure:"runtime" yaml:"runtime" json:"runtime"`
 }
 
 type SimpleConfigOptionsRuntime struct {
-	GPURequest    string `mapstructure:"gpuRequest" yaml:"gpuRequest"`
-	ServersMemory string `mapstructure:"serversMemory" yaml:"serversMemory"`
-	AgentsMemory  string `mapstructure:"agentsMemory" yaml:"agentsMemory"`
+	GPURequest    string `mapstructure:"gpuRequest,omitempty" yaml:"gpuRequest,omitempty" json:"gpuRequest,omitempty"`
+	ServersMemory string `mapstructure:"serversMemory,omitempty" yaml:"serversMemory,omitempty" json:"serversMemory,omitempty"`
+	AgentsMemory  string `mapstructure:"agentsMemory,omitempty" yaml:"agentsMemory,omitempty" json:"agentsMemory,omitempty"`
 }
 
 type SimpleConfigOptionsK3d struct {
-	Wait                       bool                 `mapstructure:"wait" yaml:"wait"`
-	Timeout                    time.Duration        `mapstructure:"timeout" yaml:"timeout"`
-	DisableLoadbalancer        bool                 `mapstructure:"disableLoadbalancer" yaml:"disableLoadbalancer"`
-	DisableImageVolume         bool                 `mapstructure:"disableImageVolume" yaml:"disableImageVolume"`
-	NoRollback                 bool                 `mapstructure:"disableRollback" yaml:"disableRollback"`
-	PrepDisableHostIPInjection bool                 `mapstructure:"disableHostIPInjection" yaml:"disableHostIPInjection"`
-	NodeHookActions            []k3d.NodeHookAction `mapstructure:"nodeHookActions" yaml:"nodeHookActions,omitempty"`
+	Wait                       bool                 `mapstructure:"wait" yaml:"wait" json:"wait"`
+	Timeout                    time.Duration        `mapstructure:"timeout" yaml:"timeout,omitempty" json:"timeout,omitempty"`
+	DisableLoadbalancer        bool                 `mapstructure:"disableLoadbalancer" yaml:"disableLoadbalancer" json:"disableLoadbalancer"`
+	DisableImageVolume         bool                 `mapstructure:"disableImageVolume" yaml:"disableImageVolume" json:"disableImageVolume"`
+	NoRollback                 bool                 `mapstructure:"disableRollback" yaml:"disableRollback" json:"disableRollback"`
+	PrepDisableHostIPInjection bool                 `mapstructure:"disableHostIPInjection" json:"disableHostIPInjection"`
+	NodeHookActions            []k3d.NodeHookAction `mapstructure:"nodeHookActions" yaml:"nodeHookActions,omitempty" json:"nodeHookActions,omitempty"`
 }
 
 type SimpleConfigOptionsK3s struct {
-	ExtraServerArgs []string `mapstructure:"extraServerArgs" yaml:"extraServerArgs"`
-	ExtraAgentArgs  []string `mapstructure:"extraAgentArgs" yaml:"extraAgentArgs"`
+	ExtraServerArgs []string `mapstructure:"extraServerArgs,omitempty" yaml:"extraServerArgs,omitempty" json:"extraServerArgs,omitempty"`
+	ExtraAgentArgs  []string `mapstructure:"extraAgentArgs,omitempty" yaml:"extraAgentArgs,omitempty" json:"extraAgentArgs,omitempty"`
 }
 
 // SimpleConfig describes the toplevel k3d configuration file.
 type SimpleConfig struct {
 	configtypes.TypeMeta `mapstructure:",squash" yaml:",inline"`
-	Name                 string                  `mapstructure:"name" yaml:"name" json:"name,omitempty"`
-	Servers              int                     `mapstructure:"servers" yaml:"servers" json:"servers,omitempty"` //nolint:lll    // default 1
-	Agents               int                     `mapstructure:"agents" yaml:"agents" json:"agents,omitempty"`    //nolint:lll    // default 0
-	ExposeAPI            SimpleExposureOpts      `mapstructure:"kubeAPI" yaml:"kubeAPI" json:"kubeAPI,omitempty"`
-	Image                string                  `mapstructure:"image" yaml:"image" json:"image,omitempty"`
-	Network              string                  `mapstructure:"network" yaml:"network" json:"network,omitempty"`
-	Subnet               string                  `mapstructure:"subnet" yaml:"subnet" json:"subnet,omitempty"`
-	ClusterToken         string                  `mapstructure:"token" yaml:"clusterToken" json:"clusterToken,omitempty"` // default: auto-generated
-	Volumes              []VolumeWithNodeFilters `mapstructure:"volumes" yaml:"volumes" json:"volumes,omitempty"`
-	Ports                []PortWithNodeFilters   `mapstructure:"ports" yaml:"ports" json:"ports,omitempty"`
-	Labels               []LabelWithNodeFilters  `mapstructure:"labels" yaml:"labels" json:"labels,omitempty"`
-	Options              SimpleConfigOptions     `mapstructure:"options" yaml:"options" json:"options,omitempty"`
-	Env                  []EnvVarWithNodeFilters `mapstructure:"env" yaml:"env" json:"env,omitempty"`
+	Name                 string                  `mapstructure:"name" yaml:"name,omitempty" json:"name,omitempty"`
+	Servers              int                     `mapstructure:"servers" yaml:"servers,omitempty" json:"servers,omitempty"` //nolint:lll    // default 1
+	Agents               int                     `mapstructure:"agents" yaml:"agents,omitempty" json:"agents,omitempty"`    //nolint:lll    // default 0
+	ExposeAPI            SimpleExposureOpts      `mapstructure:"kubeAPI" yaml:"kubeAPI,omitempty" json:"kubeAPI,omitempty"`
+	Image                string                  `mapstructure:"image" yaml:"image,omitempty" json:"image,omitempty"`
+	Network              string                  `mapstructure:"network" yaml:"network,omitempty" json:"network,omitempty"`
+	Subnet               string                  `mapstructure:"subnet" yaml:"subnet,omitempty" json:"subnet,omitempty"`
+	ClusterToken         string                  `mapstructure:"token" yaml:"clusterToken,omitempty" json:"clusterToken,omitempty"` // default: auto-generated
+	Volumes              []VolumeWithNodeFilters `mapstructure:"volumes" yaml:"volumes,omitempty" json:"volumes,omitempty"`
+	Ports                []PortWithNodeFilters   `mapstructure:"ports" yaml:"ports,omitempty" json:"ports,omitempty"`
+	Labels               []LabelWithNodeFilters  `mapstructure:"labels" yaml:"labels,omitempty" json:"labels,omitempty"`
+	Options              SimpleConfigOptions     `mapstructure:"options" yaml:"options,omitempty" json:"options,omitempty"`
+	Env                  []EnvVarWithNodeFilters `mapstructure:"env" yaml:"env,omitempty" json:"env,omitempty"`
 	Registries           struct {
 		Use    []string `mapstructure:"use" yaml:"use,omitempty" json:"use,omitempty"`
 		Create bool     `mapstructure:"create" yaml:"create,omitempty" json:"create,omitempty"`
